#!/usr/bin/env python

'''
Gimp plugin "Heal transparency"

Copyright 2010 lloyd konneker (bootch at nc.rr.com)

Version:
  1.0 lloyd konneker lkk 2010 Initial version in python.

License:

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  The GNU Public License is available at
  http://www.gnu.org/copyleft/gpl.html
  
'''

from gimpfu import *

gettext.install("resynthesizer", gimp.locale_directory, unicode=True)

def heal_transparency(timg, tdrawable, samplingRadiusParam=50, orderParam=2):

  # precondition should be enforced by Gimp according to image modes allowed.
  if not pdb.gimp_drawable_has_alpha(tdrawable):
    pdb.gimp_message("The active layer has no alpha channel to heal.")
    return
  
  pdb.gimp_image_undo_group_start(timg)
  
  # save selection for later restoration.
  # Saving selection channel makes it active, so we must save and restore the active layer
  org_selection = pdb.gimp_selection_save(timg)
  pdb.gimp_image_set_active_layer(timg, tdrawable)
  
  # alpha to selection
  pdb.gimp_image_select_item(timg, CHANNEL_OP_REPLACE, tdrawable)
  # Want the transparent, not the opaque.  
  pdb.gimp_selection_invert(timg)
  # Since transparency was probably anti-aliased (dithered with partial transpancy),
  # grow the selection to get past the dithering.
  pdb.gimp_selection_grow(timg, 1)
  # Remove the alpha from this layer. IE compose with current background color (often white.)
  # Resynthesizer won't heal transparent.
  pdb.gimp_layer_flatten(tdrawable)
  
  # Call heal selection (not the resynthesizer), which will create a proper corpus.
  # 0 = sample from all around
  pdb.python_fu_heal_selection(timg, tdrawable, samplingRadiusParam, 0, orderParam, run_mode=RUN_NONINTERACTIVE)
 
  # Restore image to initial conditions of user, except for later cleanup.
  
  # restore selection
  pdb.gimp_image_select_item(timg, CHANNEL_OP_REPLACE, org_selection)
 
  # Clean up (comment out to debug)
  pdb.gimp_image_undo_group_end(timg)


register(
  "python_fu_heal_transparency",
  N_("Removes alpha channel by synthesis.  Fill outward for edges, inward for holes."),
  "Requires separate resynthesizer plugin.",
  "Lloyd Konneker",
  "Copyright 2010 Lloyd Konneker",
  "2010",
  N_("Heal transparency..."),
  "RGBA, GRAYA",  # !!! Requires an alpha channel to heal
  [
    (PF_IMAGE, "image",       "Input image", None),
    (PF_DRAWABLE, "drawable", "Input drawable", None),
    (PF_INT, "samplingRadiusParam", _("Context sampling width (pixels):"), 50),
    (PF_OPTION, "orderParam",   _("Filling order:"), 2, [_("Random"),
      _("Inwards towards center"), _("Outwards from center") ])
  ],
  [],
  heal_transparency,
  menu="<Image>/Filters/Enhance",
  domain=("resynthesizer", gimp.locale_directory)
  )

main()



#!/usr/bin/env python

'''
Gimp plugin "Enlarge and resynthesize"

Author:
lloyd konneker
Based on smart_enlarge.scm 2000 by Paul Harrison.

Version:
1.0 lloyd konneker lkk 2010 Initial version in python.

License:

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

The GNU Public License is available at
http://www.gnu.org/copyleft/gpl.html
'''

from gimpfu import *

gettext.install("resynthesizer", gimp.locale_directory, unicode=True)

def plugin_main(image, drawable, scale_factor):
  '''
  Algorithm:
  
  Scale image up.
  Resynthesize with:
    corpus = original size image
    in map = original size image but scaled up and down to blur
    out map = scaled up image
    
  This restores the detail that scaling up looses.
  It maintains the aspect ratio of all image features.
  
  Unlike the original smart-enlarge.scm, this alters the original image.
  
  original did not accept an alpha channel
  '''

  
  temp_image1 = pdb.gimp_image_duplicate(image)  # duplicate for in map
  if not temp_image1:
    raise RuntimeError, "Failed duplicate image"
  temp_image2 = pdb.gimp_image_duplicate(image)  # duplicate for corpus
  if not temp_image2:
    raise RuntimeError, "Failed duplicate image"
  temp_layer1 = pdb.gimp_image_get_active_layer(temp_image1)
  if not temp_layer1:
    raise RuntimeError, "Failed get active layer"
  temp_layer2 = pdb.gimp_image_get_active_layer(temp_image2)
  if not temp_layer2:
    raise RuntimeError, "Failed get active layer"

  # scale input map down and back, to blur
  width = pdb.gimp_drawable_width(drawable)
  height = pdb.gimp_drawable_height(drawable)
  pdb.gimp_image_scale(temp_image1, width/scale_factor, height/scale_factor)
  pdb.gimp_image_scale(temp_image1, width, height)
    
  # scale up the image
  pdb.gimp_image_scale(image, width * scale_factor, height*scale_factor)

  # Resynthesize to restore details.
  # Note there should not be a selection. TODO
  pdb.plug_in_resynthesizer(
      image,
      drawable,
      0, 0, 0,
      temp_layer2.ID,  # corpus
      temp_layer1.ID,  # input map
      drawable.ID,     # output map is scaled up original itself
      1.0, 0.117, 8, 500) 

  pdb.gimp_image_delete(temp_image1)
  pdb.gimp_image_delete(temp_image2)


if __name__ == "__main__" :
  register(
    "python_fu_enlarge_resynthesized",
    N_("Enlarge image and synthesize to sharpen."),
    "Requires separate resynthesizer plugin",
    "Lloyd Konneker",
    "Copyright 2000 Paul Harrison, 2010 Lloyd Konneker",
    "2010",
    N_("Enlarge & sharpen..."),
    "RGB*, GRAY*",
    [
      (PF_IMAGE, "image",       "Input image", None),
      (PF_DRAWABLE, "drawable", "Input drawable", None),
      (PF_ADJUSTMENT, "scale_factor", _("Scale by:"), 2, (1, 32, 0.1))
    ],
    [],
    plugin_main,
    menu="<Image>/Filters/Enhance",
    domain=("resynthesizer", gimp.locale_directory)
    )

  main()
  
  



#!/usr/bin/env python

'''
Gimp plugin "Sharpen by resynthesis"
Requires resynthesizer plug_in (v2).

Author:
lloyd konneker (bootch at nc.rr.com)
Based on smart_enlarge.scm 2000 by Paul Harrison.

Version:
1.0 lloyd konneker lkk 2010 Initial version in python.


License:

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

The GNU Public License is available at
http://www.gnu.org/copyleft/gpl.html

'''

from gimpfu import *

gettext.install("resynthesizer", gimp.locale_directory, unicode=True)

def plugin_main(image, drawable, scale_factor):
  '''
  Algorithm:
  
  Resynthesize with:
    corpus = smaller image
    in map = smaller image but scaled up and down to blur
    out map = original image
  
  TODO undo
  
  original did not accept an alpha channel
  '''

  
  temp_image1 = pdb.gimp_image_duplicate(image)  # duplicate for in map
  if not temp_image1:
    raise RuntimeError, "Failed duplicate image"
  temp_image2 = pdb.gimp_image_duplicate(image)  # duplicate for corpus
  if not temp_image2:
    raise RuntimeError, "Failed duplicate image"
  temp_layer1 = pdb.gimp_image_get_active_layer(temp_image1)
  if not temp_layer1:
    raise RuntimeError, "Failed get active layer"
  temp_layer2 = pdb.gimp_image_get_active_layer(temp_image2)
  if not temp_layer2:
    raise RuntimeError, "Failed get active layer"

  
  width = pdb.gimp_drawable_width(drawable)
  height = pdb.gimp_drawable_height(drawable)
  
  
  # scale input image down, for corpus map
  pdb.gimp_image_scale(temp_image2, width/scale_factor, height/scale_factor)
    
  # scale input image way down, then back up to, to blur, for corpus
  # Final size is same size as corpus map.
  pdb.gimp_image_scale(temp_image1, width / (2 * scale_factor), height / (2 * scale_factor) )
  pdb.gimp_image_scale(temp_image1, width/scale_factor, height/scale_factor)

  # Resynthesize to restore details.
  # Note there should not be a selection. TODO
  pdb.plug_in_resynthesizer(
      image,
      drawable,
      0, 0, 0,
      temp_layer2.ID,  # corpus is smaller original
      temp_layer1.ID,  # input map is blurred smaller original
      drawable.ID,     # output map is original itself
      1.0, 0.117, 8, 500) 

  pdb.gimp_image_delete(temp_image1)
  pdb.gimp_image_delete(temp_image2)


register(
  "python_fu_sharpen_resynthesized",
  N_("Sharpen image by synthesis."),
  "Requires separate resynthesizer plugin.",
  "Lloyd Konneker",
  "Copyright 2000 Paul Harrison, 2010 Lloyd Konneker",
  "2010",
  N_("Sharpen by synthesis..."),
  "RGB*, GRAY*",
  [ (PF_IMAGE, "image",       "Input image", None),
    (PF_DRAWABLE, "drawable", "Input drawable", None),
    (PF_ADJUSTMENT, "sharpen_factor", _("Sharpening:"), 2, (1, 32, 0.1)),
  ],
  [],
  plugin_main,
  menu="<Image>/Filters/Enhance",
  domain=("resynthesizer", gimp.locale_directory)
  )

main()





#!/usr/bin/env python

'''
Gimp plugin.

Create new image having texture synthesized from the selection.
Works best if selection is natural (fractal).
Can work with man-made regular texture.
Works worst with man-made, structured but not regular, symbols.
Sometimes called rendering a texture.

Requires resynthesizer plug-in.

Author:
lloyd konneker, lkk, bootch at nc.rr.com

Version:
1.0 lkk 7/15/2010 Initial version
1.1 lkk 4/10/2011 Fixed a bug with layer types impacting greyscale images.

License:

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

The GNU Public License is available at
http://www.gnu.org/copyleft/gpl.html


The effect for users:
Similar to "Fill resynthesized pattern" except:
  - here the arguments are reversed: you select a texture and create a new image
    instead of selecting an area and choosing a pattern.
  - here the result is less random (since Fill resynthesized adds noise.)
Different from tiling since:
  - seamless and irregular pattern

The continuum of randomness versus speed:
  - Filters.Map.Tile is fast but has seams and regular pattern (even if you use "Make Seamless" first.)
  - Filter.Render.Texture a tile followed by tiling is seamless but still has regularity.
  - Filte.Render.Texture an entire image is slower but seamless and moderately irregular.
  - Edit.Fill with resynthesized pattern is slowest but seamless and highly irregular, unpatterned.

This filter is not tiling (instead resynthesizing) but makes 
an image that you can then use to tile with especially if
you choose the option to make the edges suitable for tiling.  

IN: The selection (or the entire active drawable) is the source of texture and is not changed.
OUT New image, possibly resized canvas, same scale and resolution.

TODO a quality setting
'''

from gimpfu import *

gettext.install("resynthesizer", gimp.locale_directory, unicode=True)

debug = False

    
def new_resized_image(image, resize_ratio):
  # Create new image resized by a ratio from *selection* in the old image
  
  (is_selection, ulx, uly, lrx, lry) = pdb.gimp_selection_bounds(image)
  if not is_selection :
    # Resynthesizer will use the entire source image as corpus.
    # Resize new image in proportion to entire source image.
    new_width = int(pdb.gimp_image_width(image) * resize_ratio)
    new_height = int(pdb.gimp_image_height(image) * resize_ratio)
    new_type = pdb.gimp_image_base_type(image)  # same as source
  else :
    # Resize new image in proportion to selection in source
    new_width = int((lrx - ulx) * resize_ratio)
    new_height = int((lry - uly) * resize_ratio)
    
  new_basetype = pdb.gimp_image_base_type(image)  # same as source
  new_layertype = pdb.gimp_drawable_type(pdb.gimp_image_get_active_layer(image))
  new_image = pdb.gimp_image_new(new_width, new_height, new_basetype)
  # !!! Note that gimp_layer_new wants a layer type, not an image basetype
  new_drawable = pdb.gimp_layer_new(new_image, new_width, new_height,
    new_layertype, "Texture", 100, NORMAL_MODE)
  pdb.gimp_image_add_layer(new_image, new_drawable, 0)
  return new_image, new_drawable
 

def display_image(image):
  try:
    gimp.Display(image)
  except gimp.error:
    pass  # If runmode is NONINTERACTIVE, expect gimp_display_new() to fail
  gimp.displays_flush()
  

def render_texture(image, drawable, resize_ratio=2, make_tile=0):
  '''
  Create a randomized texture image from the selection. 
  The image can be suited for further, seamless tiling. 
  The image is same scale and resolution but resized from the selection.
  Not undoable, no changes to the source (you can just delete the new image.)
  
  A selection in the source image is optional.
  If there is no selection, the resynthesizer will use the entire source image.
  '''
  
  # Its all or nothing, user must delete new image if not happy.
  pdb.gimp_image_undo_disable(image)
  
  '''
  Create new image, optionally resized, and display for it.
  '''
  new_image, new_drawable = new_resized_image(image, resize_ratio)
  pdb.gimp_image_undo_disable(new_image)
  if not new_drawable:
    raise RuntimeError, "Failed create layer."
  
  # If using new resynthesizer with animation for debugging
  if debug:
    display_image(new_image)
  
  '''
  copy original into temp and crop it to the selection to save memory in resynthesizer
  '''
  temp_image = pdb.gimp_image_duplicate(image)
  if not temp_image:
      raise RuntimeError, "Failed duplicate image"
  
  # Get bounds, offset of selection
  (is_selection, ulx, uly, lrx, lry) = pdb.gimp_selection_bounds(image)
  if not is_selection :
    # No need to crop.  Resynthesizer will use all if no selection.
    pass
  else :
    pdb.gimp_image_crop(temp_image, lrx - ulx, lry - uly, ulx, uly)
 
  # Don't flatten because it turns transparency to background (white usually)
  work_layer = pdb.gimp_image_get_active_layer(temp_image)
  if not work_layer:
    raise RuntimeError, "Failed get active layer"
  
  # Insure the selection is all (not necessary, resynthesizer will use all if no selection.)
  pdb.gimp_selection_all(temp_image)
  
  # Settings for making edges suitable for seamless tiling afterwards.
  # That is what these settings mean in the resynthesizer:
  # wrap context probes in the target so that edges of target will be suitable for seamless tiling.
  # I.E. treat the target as a sphere when matching.
  if make_tile :
    htile = 1
    vtile = 1
  else :
    htile = 0
    vtile = 0
  
  # Call resynthesizer
  # use_border is moot since there is no context (outside the target) in the newImage.
  # The target is the entire new image, the source is the cropped copy of the selection.
  #
  # 9 neighbors (a 3x3 patch) and 200 tries for speed, since new image is probably large
  # and source is probably natural (fractal), where quality is not important.
  
  # For version of resynthesizer with uninverted selection
  # !!! Pass -1 for ID of no layer, not None
  pdb.plug_in_resynthesizer(new_image, new_drawable, htile, vtile, 0, work_layer.ID, -1, -1, 0.0, 0.117, 9, 200)
  
  display_image(new_image)
  
  # Clean up. 
  pdb.gimp_image_delete(temp_image)
  pdb.gimp_image_undo_enable(image)
  pdb.gimp_image_undo_enable(new_image)

  return new_image

register(
  "python_fu_render_texture",
  N_("Create a new image with texture from the current image or selection. Optionally, create image edges suited for further, seamless tiling. "),
  "New image is the same scale but seamless and irregular.  Use 'Map>Tile' for less randomness. Use 'Edit>Fill resynthesized pattern' for more randomness. Requires separate resynthesizer plugin.",
  "Lloyd Konneker",
  "Copyright 2010 Lloyd Konneker",
  "2010",
  N_("_Texture..."),
  "RGB*, GRAY*",
  [
    (PF_IMAGE, "image",       "Input image", None),
    (PF_DRAWABLE, "drawable", "Input drawable", None),
    # Spinner is digital and linear, slider is analog but exponential
    (PF_SPINNER, "resize_ratio", _("Ratio of size of new image to source selection"), 2, (0.5, 10, 0.5)),
    (PF_TOGGLE, "make_tile", _("Make new image edges suitable for seamless tiling"), 0 )
  ],
  [(PF_IMAGE, "new_image", "New, synthesized texture.")],
  render_texture,
  menu="<Image>/Filters/Render",
  domain=("resynthesizer", gimp.locale_directory)
  )

main()

